<?php

namespace Bolt\Extension\Ginger\Squarepay\Controller;

use Bolt\Controller\Base;
use Silex\Application;
use Silex\ControllerCollection;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * The controller for Square backend routes.
 * 
 * Get an OAuth Access Token, and store it - it's valid for 30 days,
 * then enters 15 day grace period during which it should usually be renewed
 *
 * @author Richard Knight <rich@apewave.com>
 */
class SquareBackendController extends Base
{
    public function __construct($config)
    {
        $this->config = $config;

        // Create and configure a new API client object
        $apiConfig = new \SquareConnect\Configuration();
        $apiConfig->setAccessToken($config['sq_sandbox_token']);
        $this->apiClient = new \SquareConnect\ApiClient($apiConfig);
    }    
    
    public function addRoutes(ControllerCollection $collection)
    {
        $collection->match('/', [$this, 'squareDashboard']);
        $collection->match('/request-oauth-token', [$this, 'oauthRequestToken']);
        return $collection;
    }

    public function squareDashboard()
    {
        $params = [
            'client_id' => $this->config['sq_app_id'],
            'scope' => 'ITEMS_READ MERCHANT_PROFILE_READ PAYMENTS_READ PAYMENTS_WRITE ORDERS_READ',
        ];
        $uri = 'https://'.$this->config['sq_domain'].$this->config['sq_auth_url'];
        $query = http_build_query($params, '', '&', PHP_QUERY_RFC3986);
        $auth_link = $uri.'?'.$query;
        $context = [
            'auth_link' => $auth_link, 
            'config' => $this->config
        ];
        return $this->render('@Squareback/dashboard.twig', $context);
    }

    public function oauthRequestToken(Application $app, Request $request)
    {
        try {
            $authorizationCode = $request->query->get('code');
            if (!$authorizationCode) {
                error_log('Authorization failed!');
                throw new \Exception('Error Processing Request: Authorization failed!', 1);
            }
            $oauthToken = $this->getOAuthToken($authorizationCode);
        } catch (Exception $e) {
            error_log($e->getMessage());
            return $this->render('@Squareback/error.twig', ['error' => $e->getMessage()]);
        }
        
        // Now store the token
        $manager = $app['filesystem'];
        if ($manager->hasFilesystem('extensions_config')) {
            $filesystem = $manager->getFilesystem('extensions_config');
            $filesystem->put('.sqoatoken', $oauthToken);
        } else {
            return $this->render('@Squareback/error.twig', ['error' =>'Could not write .squotoken to app/config/extensions/']);
        }
        return $this->render('@Squareback/authorised.twig', ['token' => $oauthToken]);
    }

    // ------------------------------------------------------------------------------------------------------------------------------------------

    private function getOAuthToken($authorizationCode) {
        // Exchange the authorization code for an OAuth token
        // Create an OAuth API client
        $oauthApi = new \SquareConnect\Api\OAuthApi($this->apiClient);
        $body = new \SquareConnect\Model\ObtainTokenRequest();
      
        // Set the POST body
        $body->setClientId($this->config['sq_app_id']);
        $body->setClientSecret($this->config['sq_app_secret']);
        $body->setCode($authorizationCode);
        try {
            $result = $oauthApi->obtainToken($body);
        } catch (Exception $e) {
            error_log('Exception when calling OAuthApi->obtainToken: ' . $e->getMessage());
            throw new Exception('Error Processing Request: Token exchange failed!', 1);
        }
        return $result->getAccessToken();
    }

}
